%{
  // #define free(x)
  //#define fclose(x)

  /*
    rcompile.lex version 1.0

    lex scanner for

    Resource Compiler for Helio

    This program uses the file ui.h from the VTech Helio SDK, which
    is licensed by VTech.
    The remaining files can be distributed freely under the terms of the GPL.

    The generated scanner must be compiled with C++

    send bugs/comments to:

    James Osborn
    josborn58@attbi.com
  */

#include <string.h>
#include "ui.h"
#include "reslib.h"

#define read_boolean(x) { rboolean = &(x); yy_push_state(READBOOLEAN); }
#define read_text(x)    { rtext    = &(x); yy_push_state(READTEXT); }

#define read_type(x)      { rtype      = &(x); yy_push_state(READTYPE); }
#define read_bounds(x)    { rbounds    = &(x); yy_push_state(READBOUNDS); }
#define read_bitmap(x)    { rbitmap    = &(x); x.bitmap_data = NULL; yy_push_state(READBITMAP); }
#define read_color(x)     { rcolor     = &(x); yy_push_state(READCOLOR); }
#define read_font(x)      { rfont      = &(x); yy_push_state(READFONT); }
#define read_alignment(x) { ralignment = &(x); yy_push_state(READALIGNMENT); }

  int nobjid=0, nobjtype=0;
  USHORT tableid;
  int bitmapnum;
  int menuitem;
  int tablecolumn, tablerow, tablecell;
  int popupitem;
  int listobj, listitem;

  Form form;
  Control control;
  ControlTemplateButton button;
  ControlTemplatePushButton pushbutton;
  ControlTemplateRepeatButton repeatbutton;
  ControlTemplateCheckBox checkbox;
  ControlTemplatePopupTrigger popuptrigger;
  Field field;
  List list;
  Menu menu;
  Scrollbar scrollbar;
  Table table;
  Schline schline;
  String string;
  Bitmap bitmap;
  Textbox textbox;
  Line line;

  BYTE *rbyte;
  UBYTE *rubyte;
  SHORT *rshort;
  USHORT *rushort;
  WORD *rword;
  UWORD *ruword;
  BOOLEAN *rboolean;
  BYTE **rtext;

  BYTE *rtype;
  ObjectBounds *rbounds;
  BitmapTemplate *rbitmap;
  BYTE *bitmapfilename;
  BYTE *rcolor;
  BYTE *rfont;
  BYTE *ralignment;

  char *name = NULL;
  FILE *binfile, *hfile;

  void read_num(BYTE *x);
  void read_num(UBYTE *x);
  void read_num(SHORT *x);
  void read_num(USHORT *x);
  void read_num(WORD *x);
  void read_num(UWORD *x);
  void error(char *s);
  void write_header(int);

#define YY_USER_INIT { \
    form.form_objects_list = NULL; \
    menu.menu_items = NULL; \
  }

%}

SP  [ \t]*
NL  [ \t]*[\r]?\n
EQ  [ \t]*=
PT  (\(.*\))?[ \t]*

%x SKIP_NUM
%x READ_BYTE
%x READ_UBYTE
%x READ_SHORT
%x READ_USHORT
%x READ_WORD
%x READ_UWORD
%x READBOOLEAN
%x READTEXT
%x READTYPE
%x READBOUNDS
%x READBITMAP
%x READBITMAPFILE
%x READCOLOR
%x READFONT
%x READALIGNMENT
%x FORM
%x FORM_STYLE
%x CONTROL
%x CONTROL_STYLE
%x CONTROL_SUBTYPE
%x CONTROL_ATTR
%x CONTROL_ADD_BUTTON
%x CONTROL_ADD_PUSHBUTTON
%x CONTROL_ADD_CHECKBOX
%x CONTROL_ADD_POPUPTRIGGER
%x FIELD
%x FIELD_STYLE
%x FIELD_BACK_LINE
%x FIELD_ATTR
%x LIST
%x LIST_STYLE
%x LIST_ATTR
%x MENU
%x SCROLLBAR
%x SCROLLBAR_STYLE
%x SCROLLBAR_TYPE
%x SCROLLBAR_ATTR
%x TABLE
%x TABLE_STYLE
%x TABLE_DATA_TYPE
%x TABLE_ATTR
%x SCHLINE
%x SCHLINE_ATTR
%x STRING
%x STRING_STYLE
%x STRING_ATTR
%x BITMAP
%x BITMAP_STYLE
%x BITMAP_ATTR
%x TEXTBOX
%x TEXTBOX_STYLE
%x TEXTBOX_BACK_LINE
%x TEXTBOX_ATTR
%x LINE
%x LINE_STYLE
%option caseless
%option stack
%option noyy_top_state
%%

<*>{
  ^{NL}               /* skip blank line */
  ^"//"[^\n]*\n       /* skip comment */
  ^"~F"[A-Z0-9]+{NL}  /* skip */
  ^NAME{EQ}           read_text(name);
}

{NL}             /* finish newline from end resource type */
^#FORM{NL}       { printf("FORM"); BEGIN(FORM); }
^#CONTROL{NL}    { printf("CONTROL"); control.control_text = NULL; BEGIN(CONTROL); }
^#FIELD{NL}      { printf("FIELD"); BEGIN(FIELD); }
^#LIST{NL}       { printf("LIST"); BEGIN(LIST); }
^#MENU{NL}       { printf("MENU"); BEGIN(MENU); }
^#SCROLLBAR{NL}  { printf("SCROLLBAR"); scrollbar.scrollbar_text = NULL; BEGIN(SCROLLBAR); }
^#TABLE{NL}      { printf("TABLE"); BEGIN(TABLE); }
^#SCHLINE{NL}    { printf("SCHLINE"); BEGIN(SCHLINE); }
^#STRING{NL}     { printf("STRING"); BEGIN(STRING); }
^#BITMAP{NL}     { printf("BITMAP"); BEGIN(BITMAP); }
^#TEXTBOX{NL}    { printf("TEXTBOX"); BEGIN(TEXTBOX); }
^#LINE{NL}       { printf("LINE"); BEGIN(LINE); }

<SKIP_NUM>{
  [+-]?[0-9]*{NL}  yy_pop_state();
}

<READ_BYTE>{
  [+-]?[0-9]*{NL}  { *rbyte = (BYTE)strtol(yytext, NULL, 10); yy_pop_state(); }
  .                error("READ_BYTE");
}

<READ_UBYTE>{
  [0-9]*{NL}  { *rubyte = (UBYTE)strtol(yytext, NULL, 10); yy_pop_state(); }
  .           error("READ_UBYTE");
}

<READ_SHORT>{
  NO_SELECTION{NL} { *rshort = 0; yy_pop_state(); }
  [+-]?[0-9]*{NL}  { *rshort = (SHORT)strtol(yytext,NULL,10); yy_pop_state(); }
  .                error("READ_SHORT");
}

<READ_USHORT>{
  [0-9]*{NL}  { *rushort = (USHORT)strtol(yytext, NULL, 10); yy_pop_state(); }
  .           error("READ_USHORT");
}

<READ_WORD>{
  [+-]?[0-9]*{NL}  {
    *rword = (WORD)strtol(yytext, NULL, 10);
    if(*rword<0) *rword += 0x10000; yy_pop_state();
  }
  .  error("READ_WORD");
}

<READ_UWORD>{
  [0-9]*{NL}  { *ruword = (UWORD)strtol(yytext, NULL, 10); yy_pop_state(); }
  .           error("READ_UWORD");
}

<READBOOLEAN>{
  0+{NL}         |
  F[A-Z]*{NL}    { *rboolean = 0; yy_pop_state(); }
  0*[1-9]+0*{NL} |
  T[A-Z]*{NL}    { *rboolean = 1; yy_pop_state(); }
  .              error("READBOOLEAN");
}

<READTEXT>{
  [^\r\n]*\n {
    *rtext = (char *)malloc(yyleng);
    memcpy(*rtext, yytext, yyleng-1);
    (*rtext)[yyleng-1] = '\000';
    yy_pop_state();
  }
  [^\r\n]*\r\n {
    *rtext = (char *)malloc(yyleng-1);
    memcpy(*rtext, yytext, yyleng-2);
    (*rtext)[yyleng-2] = '\000';
    yy_pop_state();
  }
}

<READTYPE>{
  FORM{NL}       { *rtype = 0;  yy_pop_state(); }
  CONTROL{NL}    { *rtype = 1;  yy_pop_state(); }
  FIELD{NL}      { *rtype = 2;  yy_pop_state(); }
  LIST{NL}       { *rtype = 3;  yy_pop_state(); }
  MENU{NL}       { *rtype = 4;  yy_pop_state(); }
  SCROLLBAR{NL}  { *rtype = 5;  yy_pop_state(); }
  TABLE{NL}      { *rtype = 6;  yy_pop_state(); }
  SCHLINE{NL}    { *rtype = 7;  yy_pop_state(); }
  STRING{NL}     { *rtype = 8;  yy_pop_state(); }
  BITMAP{NL}     { *rtype = 9;  yy_pop_state(); }
  TEXTBOX{NL}    { *rtype = 10; yy_pop_state(); }
  LINE{NL}       { *rtype = 11; yy_pop_state(); }
  .              error("READTYPE");
}

<READBOUNDS>{
  X{EQ}       { yy_pop_state(); read_num(&rbounds->xcoord); }
  Y{EQ}       { yy_pop_state(); read_num(&rbounds->ycoord); }
  WIDTH{EQ}   { yy_pop_state(); read_num(&rbounds->width); }
  HEIGHT{EQ}  { yy_pop_state(); read_num(&rbounds->height); }
  .           error("READBOUNDS");
}

<READBITMAP>{
  X{EQ}       { yy_pop_state(); read_num(&rbitmap->xcoord); }
  Y{EQ}       { yy_pop_state(); read_num(&rbitmap->ycoord); }
  WIDTH{EQ}   { yy_pop_state(); read_num(&rbitmap->width); }
  HEIGHT{EQ}  { yy_pop_state(); read_num(&rbitmap->height); }
  FILE{EQ}    { yy_pop_state(); ++bitmapnum; yy_push_state(READBITMAPFILE); }
  .           error("READBITMAP");
}

<READBITMAPFILE>{
  Q_TWO_BIT{SP},{SP} {
    rbitmap->compressed = 0;
    rbitmap->quantisation = 2;
    read_text(bitmapfilename);
  }
  Q_FOUR_BIT{SP},{SP} {
    rbitmap->compressed = 0;
    rbitmap->quantisation = 4;
    read_text(bitmapfilename);
  }
  ^. {
    unput(yytext[0]);
    yy_set_bol(1);
    load_bitmap_template(rbitmap, bitmapfilename);
    yy_pop_state();
  }
}

<READCOLOR>{
  COLOR_W[A-Z0-9]*{NL}  { *rcolor = 0; yy_pop_state(); }
  COLOR_GREY1{NL}       { *rcolor = 1; yy_pop_state(); }
  COLOR_GREY2{NL}       { *rcolor = 2; yy_pop_state(); }
  COLOR_B[A-Z0-9]*{NL}  { *rcolor = 3; yy_pop_state(); }
  .                     error("READCOLOR");
}

<READFONT>{
  SMALL_FONT{NL}   |
  NORMAL_FONT{NL}  { *rfont = 0; yy_pop_state(); }
  NORMAL{NL}       { *rfont = 1; yy_pop_state(); }
  MEDIUM_FONT{NL}  { *rfont = 1; yy_pop_state(); }
  LARGE_FONT{NL}   { *rfont = 2; yy_pop_state(); }
  .                error("READFONT");
}

<READALIGNMENT>{
  LEFT{NL}    { *ralignment = 0; yy_pop_state(); }
  CENTRE{NL}  { *ralignment = 1; yy_pop_state(); }
  RIGHT{NL}   { *ralignment = 2; yy_pop_state(); }
  {NL}        { *ralignment = 0; yy_pop_state(); }
  .           error("READALIGNMENT");
}

<FORM>{
  ^I[D]?{EQ}                     read_num(&form.identification.ui_object_id);
  ^{PT}OBJECT\.TYPE{EQ}FORM{NL}  {
    printf("      ID=%-4i", form.identification.ui_object_id);
    form.identification.ui_object_type = 0;
    form.form_dialog_title = NULL;
    form.form_bitmap.bitmap_data = NULL;
  }
  ^{PT}BOUNDS\.                  read_bounds(form.bounds);
  ^{PT}FOCUSED_OBJECT{EQ}        read_num(&form.form_focus);
  ^{PT}FORM_STYLE{EQ}            yy_push_state(FORM_STYLE);
  ^{PT}FORM_TITLE{EQ}            read_text(form.form_dialog_title);
  ^{PT}FORM_BITMAP\.             read_bitmap(form.form_bitmap);
  ^{PT}NO_OF_OBJECTS{EQ} {
    nobjid = nobjtype = 0;
    read_num(&form.form_num_objects);
  }
  ^{PT}UIOBJECT.ID{EQ} |
  ^{PT}OBJECT_ID{EQ} {
    if((nobjid==0)&&(nobjtype==0))
      form.form_objects_list = (FormObjectsList *)
	realloc(form.form_objects_list,
		form.form_num_objects*sizeof(FormObjectsList));
    read_num(&form.form_objects_list[nobjid++].object_id);
  }
  ^{PT}UIOBJECT.TYPE{EQ} |
  ^{PT}OBJECT_TYPE{EQ} {
    if((nobjid==0)&&(nobjtype==0))
      form.form_objects_list = (FormObjectsList *)
	realloc(form.form_objects_list,
		form.form_num_objects*sizeof(FormObjectsList));
    read_type(form.form_objects_list[nobjtype++].object_type);
  }
  ^#END_FORM  {
    write_form(&form, binfile);
    write_header((int)form.identification.ui_object_id);
    {int len; len=ftell(binfile); printf(" end=0x%06x %i\n",len,len);}
    BEGIN(INITIAL);
  }
  .  error("FORM");
}

<FORM_STYLE>{
  BACKGROUND{NL}         { form.form_style = 0; yy_pop_state(); }
  NORMAL{NL}             { form.form_style = 1; yy_pop_state(); }
  BITMAP_DIALOG{NL}      { form.form_style = 2; yy_pop_state(); }
  NON_BITMAP_DIALOG{NL}  { form.form_style = 3; yy_pop_state(); }
  .                      error("FORM_STYLE");
}

<CONTROL>{
  ^I[D]?{EQ}                        read_num(&control.identification.ui_object_id);
  ^{PT}OBJECT\.TYPE{EQ}CONTROL{NL}  {
    printf("   ID=%-4i",control.identification.ui_object_id);
    control.identification.ui_object_type = 1;
    control.control_text = NULL;
    repeatbutton.repeat_bitmap.bitmap_data = NULL;
  }
  ^{PT}RELATED_TABLE_ID{EQ}         read_num(&tableid);
  ^{PT}BOUNDS\.                     read_bounds(control.bounds);
  ^{PT}CONTROL_STYLE{EQ}            yy_push_state(CONTROL_STYLE);
  ^{PT}CONTROL_SUBTYPE{EQ}          yy_push_state(CONTROL_SUBTYPE);
  ^{PT}CONTROL_TEXT{EQ}             read_text(control.control_text);
  ^{PT}TEXT(_ALIGN)?MENT{EQ}        read_alignment(control.text_alignment);
  ^{PT}CONTROL_ATTR\.               yy_push_state(CONTROL_ATTR);
  ^{PT}CONTROL_ADD(_ATTR)?\.  {
    switch(control.control_style) {
    case 0: yy_push_state(CONTROL_ADD_BUTTON); break;
    case 1: yy_push_state(CONTROL_ADD_PUSHBUTTON); break;
    case 3: yy_push_state(CONTROL_ADD_CHECKBOX); break;
    case 4: yy_push_state(CONTROL_ADD_POPUPTRIGGER); break;
    default: fprintf(stderr,"\ncase %i\n",control.control_style); error("CONTROL_ADD");
    }
  }
  ^{PT}CONTROL_BITMAP\.  read_bitmap(repeatbutton.repeat_bitmap);
  ^#END_CONTROL {
    switch(control.control_style) {
    case 0: write_control(&control, (void*)&button, tableid, binfile); break;
    case 1: write_control(&control, (void*)&pushbutton, tableid, binfile); break;
    case 2: write_control(&control, (void*)&repeatbutton, tableid, binfile); break;
    case 3: write_control(&control, (void*)&checkbox, tableid, binfile); break;
    case 4: write_control(&control, (void*)&popuptrigger, tableid, binfile); break;
    default: fprintf(stderr,"\ncase %i\n",control.control_style); error("END_CONTROL");
    }
    write_header((int)control.identification.ui_object_id);
    {int len; len=ftell(binfile); printf(" end=0x%06x %i\n",len,len);}
    BEGIN(INITIAL);
  }
  .  error("CONTROL");
}

<CONTROL_STYLE>{
  BUTTON{NL}        { control.control_style = 0; yy_pop_state(); }
  PUSH_BUTTON{NL}   { control.control_style = 1; yy_pop_state(); }
  REPEAT_BUTTON{NL} { control.control_style = 2; yy_pop_state(); }
  CHECKBOX{NL}      { control.control_style = 3; yy_pop_state(); }
  POPUP_TRIGGER{NL} { control.control_style = 4; yy_pop_state(); }
  .                 error("CONTROL_STYLE");
}

<CONTROL_SUBTYPE>{
  [A-Z_]*/[0-9]  { yy_pop_state(); read_num(&control.control_subtype); }
  {NL}           { control.control_subtype = 0; yy_pop_state(); }
  .              error("CONTROL_SUBTYPE");
}

<CONTROL_ATTR>{
  ENABLE{EQ}   { yy_pop_state(); read_boolean(control.control_attr.control_enable); }
  VISIBLE{EQ}  { yy_pop_state(); read_boolean(control.control_attr.control_visible); }
  ACTIVE{EQ}   { yy_pop_state(); read_boolean(control.control_attr.control_active); }
  .                    error("CONTROL_ATTR");
}

<CONTROL_ADD_BUTTON>{
  RADIUS{EQ}     { yy_pop_state(); read_num(&button.button_radius); }
  COLOR_ON{EQ}   { yy_pop_state(); read_color(button.button_color_on); }
  COLOR_OFF{EQ}  { yy_pop_state(); read_color(button.button_color_off); }
  .              error("CONTROL_ADD_BUTTON");
}

<CONTROL_ADD_PUSHBUTTON>{
  COLOR_ON{EQ}      { yy_pop_state(); read_color(pushbutton.push_button_color_on); }
  COLOR_OFF{EQ}     { yy_pop_state(); read_color(pushbutton.push_button_color_off); }
  GROUP_ID{EQ}      { yy_pop_state(); read_num(&pushbutton.push_button_group_id); }
  CONTROL_VALUE{EQ} { yy_pop_state(); read_boolean(pushbutton.control_value); }
  RADIUS{EQ}        { yy_pop_state(); read_num(&pushbutton.push_button_radius); }
  .                 error("CONTROL_ADD_PUSHBUTTON");
}

<CONTROL_ADD_CHECKBOX>{
  {PT}(CONTROL_ADD\.)?GROUP_ID{EQ}  {
    read_num(&checkbox.checkbox_group_id);
    bitmapnum = 1;
    checkbox.checkbox_bitmap1.bitmap_data = NULL;
    checkbox.checkbox_bitmap2.bitmap_data = NULL;
  }
  {PT}(CONTROL_ADD\.)?CONTROL_VALUE{EQ}  read_boolean(checkbox.control_value);
  {PT}CONTROL_BITMAP\. {
    if(bitmapnum==1) read_bitmap(checkbox.checkbox_bitmap1);
    if(bitmapnum==2) read_bitmap(checkbox.checkbox_bitmap2);
  }
  ^. {
    if(bitmapnum==3) {
      unput(yytext[0]);
      yy_set_bol(1);
      yy_pop_state();
    } else {
      error("CONTROL_ADD_CHECKBOX");
    }
  }
  .  error("CONTROL_ADD_CHECKBOX");
}

<CONTROL_ADD_POPUPTRIGGER>{
  RADIUS{EQ}         { yy_pop_state(); read_num(&popuptrigger.popup_radius); }
  NUM_(OF_)?OBJECTS{EQ} { popupitem = 0; yy_pop_state(); read_num(&popuptrigger.popup_num_objects); }
  SELECTED_ITEM{EQ}  { yy_pop_state(); read_num(&popuptrigger.popup_selected_item); }
  TOP_ITEM_NUM{EQ}   { yy_pop_state(); read_num(&popuptrigger.popup_top_item_num); }
  ITEM[0-9A-Z_]*{EQ}  {
    if(popupitem==0) {
      int i;
      popuptrigger.popup_items_list = (BYTE **)malloc(popuptrigger.popup_num_objects*sizeof(BYTE*));
      for(i=0; i<popuptrigger.popup_num_objects; ++i) {
	popuptrigger.popup_items_list[i] = NULL;
      }
    }
    yy_pop_state();
    read_text(popuptrigger.popup_items_list[popupitem++]);
  }
  .                  error("CONTROL_ADD_POPUPTRIGGER");
}

<FIELD>{
  ^I[D]?{EQ}                                 read_num(&field.identification.ui_object_id);
  ^{PT}OBJECT\.TYPE{EQ}FIELD{NL} {
    printf("     ID=%-4i",field.identification.ui_object_id);
    field.identification.ui_object_type = 2;
    field.field_string = NULL;
  }
  ^{PT}RELATED_TABLE_ID{EQ}                  read_num(&tableid);
  ^{PT}BOUNDS\.                              read_bounds(field.bounds);
  ^{PT}FIELD_TEXT{EQ}                      read_text(field.field_string);
  ^{PT}FIELD_STYLE{EQ}                     yy_push_state(FIELD_STYLE);
  ^{PT}FIELD_BACK_LINE{EQ}                 yy_push_state(FIELD_BACK_LINE);
  ^{PT}FIELD_FONT{EQ}                      read_font(field.field_font_id);
  ^{PT}FIELD_FONT_COLOR{EQ}                read_color(field.field_font_color);
  ^{PT}FIELD_BACKGROUND_COLOR{EQ}          read_color(field.field_background_color);
  ^{PT}FIELD_TEXTMENT{EQ}                  read_alignment(field.field_text_alignment);
  ^{PT}FIELD_MAX_NUM_CHARS{EQ}             read_num(&field.field_max_chars);
  ^{PT}FIELD_TOP_LINE_NUM{EQ}              read_num(&field.field_top_line_num);
  ^{PT}FIELD_INSERT_PT_CHAR_POS{EQ}        read_num(&field.field_insert_pt_char_pos);
  ^{PT}FIELD_HIGHLIGHT_START{EQ}           read_num(&field.field_highlight_start_char);
  ^{PT}FIELD_HIGHLIGHT_END{EQ}             read_num(&field.field_highlight_end_char);
  ^{PT}FIELD_ATTR\.                        yy_push_state(FIELD_ATTR);
  ^#END_FIELD {
    write_field(&field, tableid, binfile);
    write_header((int)field.identification.ui_object_id);
    {int len; len=ftell(binfile); printf(" end=0x%06x %i\n",len,len);}
    BEGIN(INITIAL);
  }
  .  error("FIELD");
}

<FIELD_STYLE>{
  [A-Z_]*/[0-9]  { yy_pop_state(); read_num(&field.field_style); }
  .              error("FIELD_STYLE");
}

<FIELD_BACK_LINE>{
  WITHOUT[A-Z_]*{NL}  |
  NO[A-Z_]*{NL}       { field.field_back_line = 0; yy_pop_state(); }
  DOT[A-Z_]*{NL}      { field.field_back_line = 1; yy_pop_state(); }
  GREY[A-Z_]*{NL}     { field.field_back_line = 2; yy_pop_state(); }
  .                   error("FIELD_BACK_LINE");
}

<FIELD_ATTR>{
  FIELD_ACTIVE{EQ}            { yy_pop_state(); read_boolean(field.field_attr.field_active); }
  FIELD_ENABLE{EQ}            { yy_pop_state(); read_boolean(field.field_attr.field_enable); }
  FIELD_DIRTY{EQ}             { yy_pop_state(); read_boolean(field.field_attr.field_dirty); }
  FIELD_HIGHLIGHT{EQ}         { yy_pop_state(); read_boolean(field.field_attr.field_highlight); }
  FIELD_INSERT_PT_VISIBLE{EQ} { yy_pop_state(); read_boolean(field.field_attr.field_insert_pt_visible);}
  FIELD_SCROLLBAR{EQ}         { yy_pop_state(); read_boolean(field.field_attr.field_scrollbar); }
  FIELD_FULL_SIZE{EQ}         { yy_pop_state(); read_boolean(field.field_attr.field_full_size); }
  FIELD_VISIBLE{EQ}           { yy_pop_state(); read_boolean(field.field_attr.field_visible); }
  .                           error("FIELD_ATTR");
}

<LIST>{
  ^I[D]?{EQ}                             read_num(&list.identification.ui_object_id);
  ^{PT}OBJECT\.TYPE{EQ}LIST{NL} {
    printf("      ID=%-4i",list.identification.ui_object_id);
    list.identification.ui_object_type = 3;
  }
  ^{PT}RELATED_TABLE_ID{EQ}              read_num(&tableid);
  ^{PT}BOUNDS\.                          read_bounds(list.bounds);
  ^{PT}LIST_NUM_RELATED_LIST_OBJECT{EQ}  { listobj = 0; read_num(&list.list_num_related_list); }
  ^{PT}LIST_RELATED_OBJECT_?[0-9]*{EQ} {
    if(listobj==0) {
      list.list_related_list_id = (ObjectID *)malloc(list.list_num_related_list*sizeof(ObjectID));
    }
    if(listobj<list.list_num_related_list) {
      read_num(&list.list_related_list_id[listobj++]);
    } else {
      yy_push_state(SKIP_NUM);
    }
  }
  ^{PT}LIST_TOTAL_NUM_ITEMS{EQ}          { listitem=0; read_num(&list.list_total_num_items); }
  ^{PT}LIST_NUM_ITEMS_ON_DISPLAY{EQ}     read_num(&list.list_num_items_on_display);
  ^{PT}LIST_TOP_ITEM_NUM{EQ}             read_num(&list.list_top_item_num);
  ^{PT}LIST_TEXT(_ALIGN)?MENT{EQ}        read_alignment(list.list_text_alignment);
  ^{PT}LIST_SELECTED_ITEM{EQ}            read_num(&list.list_selected_item);
  ^{PT}LIST_STYLE{EQ}                    yy_push_state(LIST_STYLE);
  ^{PT}LIST_TEXT_COLOR{EQ}               read_color(list.list_text_color);
  ^{PT}LIST_BACKGROUND_COLOR{EQ}         read_color(list.list_bg_color);
  ^{PT}LIST_ITEM[A-Z0-9_]*{EQ} {
    if(listitem==0) {
      int i;
      list.list_items = (BYTE**)malloc(list.list_total_num_items*sizeof(BYTE*));
      for(i=0; i<list.list_total_num_items; ++i)
	list.list_items[i] = NULL;
    }
    if(listitem<list.list_total_num_items) {
      read_text(list.list_items[listitem++]);
    }
  }
  ^{PT}LIST_ATTR\.                       yy_push_state(LIST_ATTR);
  ^#END_LIST {
    write_list(&list, tableid, binfile);
    write_header((int)list.identification.ui_object_id);
    {int len; len=ftell(binfile); printf(" end=0x%06x %i\n",len,len);}
    BEGIN(INITIAL);
  }
  .  error("LIST");
}

<LIST_STYLE>{
  [A-Z_]*/[0-9]  { yy_pop_state(); read_num(&list.list_style); }
  .              error("LIST_STYLE");
}

<LIST_ATTR>{
  LIST_ENABLE{EQ}       { yy_pop_state(); read_boolean(list.list_attr.list_enable); }
  LIST_VISIBLE{EQ}      { yy_pop_state(); read_boolean(list.list_attr.list_visible); }
  LIST_ACTIVE{EQ}       { yy_pop_state(); read_boolean(list.list_attr.list_active); }
  LIST_SET_SCROLL{EQ}   { yy_pop_state(); read_boolean(list.list_attr.list_set_scroll); }
  LIST_SYNCHRONOUS{EQ}  { yy_pop_state(); read_boolean(list.list_attr.list_synchronous); }
  .                     error("LIST_ATTR");
}

<MENU>{
  ^I[D]?{EQ}                             read_num(&menu.identification.ui_object_id);
  ^{PT}OBJECT\.TYPE{EQ}MENU{NL} {
    printf("      ID=%-4i",menu.identification.ui_object_id);
    menu.identification.ui_object_type = 4;
  }
  ^{PT}BOUNDS\.                          read_bounds(menu.bounds);
  ^{PT}MENU_NUM_ITEMS{EQ}                { menuitem = 0; read_num(&menu.menu_num_items); }
  ^{PT}MENU_SELECTED_ITEM{EQ}            read_num(&menu.menu_selected_item);
  ^{PT}MENU_ITEMS?[0-9]*{EQ} {
    if(menuitem==0) {
      int i;
      menu.menu_items = (BYTE **)realloc(menu.menu_items, menu.menu_num_items*sizeof(BYTE*));
      for(i=0; i<menu.menu_num_items; ++i) menu.menu_items[i] = NULL;
    }
    read_text(menu.menu_items[menuitem++]);
  }
  ^{PT}MENU_VISIBLE{EQ}                  read_boolean(menu.menu_attr.menu_visible);
  ^#END_MENU {
    write_menu(&menu, binfile);
    write_header((int)menu.identification.ui_object_id);
    {int len; len=ftell(binfile); printf(" end=0x%06x %i\n",len,len);}
    BEGIN(INITIAL);
  }
  .  error("MENU");
}

<SCROLLBAR>{
  ^I[D]?{EQ}                          read_num(&scrollbar.identification.ui_object_id);
  ^{PT}OBJECT\.TYPE{EQ}SCROLLBAR{NL} {
    printf(" ID=%-4i",scrollbar.identification.ui_object_id);
    scrollbar.identification.ui_object_type = 5;
    scrollbar.scrollbar_arrow1.bitmap_data = NULL;
    scrollbar.scrollbar_arrow2.bitmap_data = NULL;
    scrollbar.scrollbar_text = NULL;
  }
  ^{PT}RELATED_TABLE_ID{EQ}           read_num(&tableid);
  ^{PT}BOUNDS\.                       read_bounds(scrollbar.bounds);
  ^{PT}SCROLLBAR_MAX{EQ}              read_num(&scrollbar.scrollbar_max);
  ^{PT}SCROLLBAR_MIN{EQ}              read_num(&scrollbar.scrollbar_min);
  ^{PT}SCROLLBAR_CURRENT_VALUE{EQ}    read_num(&scrollbar.scrollbar_value);
  ^{PT}SCROLLBAR_PAGESIZE{EQ}         read_num(&scrollbar.scrollbar_pagesize);
  ^{PT}SCROLLBAR_TOTAL_NUM_LINES{EQ}  read_num(&scrollbar.scrollbar_total_num_lines);
  ^{PT}SCROLLBAR_STYLE{EQ}            yy_push_state(SCROLLBAR_STYLE);
  ^{PT}SCROLLBAR_TYPE{EQ}             yy_push_state(SCROLLBAR_TYPE);
  ^{PT}SCROLLBAR_ARROW1_BITMAP\.      read_bitmap(scrollbar.scrollbar_arrow1);
  ^{PT}SCROLLBAR_ARROW2_BITMAP\.      read_bitmap(scrollbar.scrollbar_arrow2);
  ^{PT}STRING_TEXT{EQ}                |
  ^{PT}SCROLLBAR_TEXT{EQ}             read_text(scrollbar.scrollbar_text);
  ^{PT}SCROLLBAR_ATTR\.               yy_push_state(SCROLLBAR_ATTR);
  ^#END_SCROLLBAR {
    write_scrollbar(&scrollbar, tableid, binfile);
    write_header((int)scrollbar.identification.ui_object_id);
    {int len; len=ftell(binfile); printf(" end=0x%06x %i\n",len,len);}
    BEGIN(INITIAL);
  }
  .  error("SCROLLBAR");
}

<SCROLLBAR_STYLE>{
  SCROLLBAR_STYLE_0{NL} { scrollbar.scrollbar_style = 0; yy_pop_state(); }
  VERTICAL{NL}          { scrollbar.scrollbar_style = 0; yy_pop_state(); }
  HORIZONTAL{NL}        { scrollbar.scrollbar_style = 1; yy_pop_state(); }
  .                     error("SCROLLBAR_STYLE");
}

<SCROLLBAR_TYPE>{
  [A-Z_]*/[0-9]   { yy_pop_state(); read_num(&scrollbar.scrollbar_type); }
  VERTICAL{NL}    { scrollbar.scrollbar_type = 0; yy_pop_state(); }
  HORIZONTAL{NL}  { scrollbar.scrollbar_type = 1; yy_pop_state(); }
  .               error("SCROLLBAR_TYPE");
}

<SCROLLBAR_ATTR>{
  SCROLLBAR_ENABLE{EQ}   { yy_pop_state(); read_boolean(scrollbar.scrollbar_attr.scrollbar_enable); }
  SCROLLBAR_VISIBLE{EQ}  { yy_pop_state(); read_boolean(scrollbar.scrollbar_attr.scrollbar_visible); }
  SCROLLBAR_ACTIVE{EQ}   { yy_pop_state(); read_boolean(scrollbar.scrollbar_attr.scrollbar_active); }
  .                      error("SCROLLBAR_ATTR");
}

<TABLE>{
  ^I[D]?{EQ}                             read_num(&table.identification.ui_object_id);
  ^{PT}OBJECT\.TYPE{EQ}TABLE{NL} {
    printf("     ID=%-4i",table.identification.ui_object_id);
    table.identification.ui_object_type = 6;
    tablecell = -1;
  }
  ^{PT}BOUNDS\.                          read_bounds(table.bounds);
  ^{PT}TABLE_NUM_COLUMN{EQ}              { tablecolumn = 0; read_num(&table.table_num_column); }
  ^{PT}TABLE_NUM_ROW{EQ}                 { tablerow = 0; read_num(&table.table_num_row); }
  ^{PT}TABLE_COLUMN_WIDTH[0-9]*{EQ} {
    if(tablecolumn==0) {
      table.table_column_width = (SHORT*)malloc(table.table_num_column*sizeof(SHORT));
    }
    read_num(&table.table_column_width[tablecolumn++]);
  }
  ^{PT}TABLE_ROW_HEIGHT[0-9]*{EQ} {
    if(tablerow==0) {
      table.table_row_height = (SHORT*)malloc(table.table_num_row*sizeof(SHORT));
    }
    read_num(&table.table_row_height[tablerow++]);
  }
  ^{PT}TABLE_TOP_ROW_NUM{EQ}       read_num(&table.table_top_row_num);
  ^{PT}TABLE_LEFT_COL_NUM{EQ}      read_num(&table.table_left_col_num);
  ^{PT}TABLE_STYLE{EQ}             yy_push_state(TABLE_STYLE);
  ^{PT}TABLE_BACKGROUND_COLOR{EQ}  {
    if(tablecell<0) {
      read_color(table.table_bg_color);
    } else {
      read_color(table.table_item_ptr[tablecell]->table_text_bg_color);
    }
  }
  ^{PT}TABLE_CELL_BITMAP.  read_bitmap(table.table_cell_bitmap);
  ^{PT}TABLE_DATA_TYPE{EQ} {
    if(tablecell<0) {
      int i, num;
      num = table.table_num_column*table.table_num_row;
      table.table_item_ptr = (TableItems **)malloc(num*sizeof(TableItems *));
      table.table_item_ptr[0] = (TableItems *)malloc(num*sizeof(TableItems));
      for(i=1; i<num; ++i) table.table_item_ptr[i] = table.table_item_ptr[i-1] + 1;
    }
    ++tablecell;
    table.table_item_ptr[tablecell]->table_text = NULL;
    yy_push_state(TABLE_DATA_TYPE);
  }
  ^{PT}TABLE_UI_OBJECT_ID{EQ}  read_num(&table.table_item_ptr[tablecell]->table_item_ui_id);
  ^{PT}TABLE_FONT{EQ}          read_font(table.table_item_ptr[tablecell]->table_font);
  ^{PT}TABLE_VALUE{EQ}         read_num(&table.table_item_ptr[tablecell]->table_value);
  ^{PT}TABLE_TEXTMENT{EQ}      read_alignment(table.table_item_ptr[tablecell]->table_display_alignment);
  ^{PT}TABLE_FONT_COLOR{EQ}    read_color(table.table_item_ptr[tablecell]->table_text_color);
  ^{PT}TABLE_CELL_HIGHLIGHT{EQ}  read_boolean(table.table_item_ptr[tablecell]->table_cell_highlight);
  ^{PT}TABLECELL_HAS_BITMAP{EQ}  read_boolean(table.table_item_ptr[tablecell]->table_cell_has_bitmap);
  ^{PT}TABLE_TEXT{EQ}          read_text(table.table_item_ptr[tablecell]->table_text);
  ^{PT}TABLE_ATTR\.            yy_push_state(TABLE_ATTR);
  ^#END_TABLE {
    write_table(&table, binfile);
    write_header((int)table.identification.ui_object_id);
    {int len; len=ftell(binfile); printf(" end=0x%06x %i\n",len,len);}
    BEGIN(INITIAL);
  }
  .  error("TABLE");
}

<TABLE_STYLE>{
  [A-Z_]*/[0-9]       { yy_pop_state(); read_num(&table.table_style); }
  .                   error("TABLE_STYLE");
}

<TABLE_DATA_TYPE>{
  TABLE_TEXT{NL}       { table.table_item_ptr[tablecell]->table_data_type = 0; yy_pop_state(); }
  TABLE_VALUE{NL}      { table.table_item_ptr[tablecell]->table_data_type = 1; yy_pop_state(); }
  TABLE_UI_OBJECT{NL}  { table.table_item_ptr[tablecell]->table_data_type = 2; yy_pop_state(); }
  .                    error("TABLE_DATA_TYPE");
}

<TABLE_ATTR>{
  TABLE_ENABLE{EQ}           { yy_pop_state(); read_boolean(table.table_attr.table_enable); }
  TABLE_SCROLLBAR{EQ}        { yy_pop_state(); read_boolean(table.table_attr.table_scrollbar); }
  TABLE_ACTIVE{EQ}           { yy_pop_state(); read_boolean(table.table_attr.table_active); }
  TABLE_HIGHLIGHT_ENABLE{EQ} { yy_pop_state(); read_boolean(table.table_attr.table_highlight_enable); }
  TABLE_VISIBLE{EQ}          { yy_pop_state(); read_boolean(table.table_attr.table_visible); }
  .                          error("TABLE_ATTR");
}

<SCHLINE>{
  ^I[D]?{EQ}                        read_num(&schline.identification.ui_object_id);
  ^{PT}OBJECT\.TYPE{EQ}SCHLINE{NL} {
    printf("   ID=%-4i",schline.identification.ui_object_id);
    schline.identification.ui_object_type = 7;
    schline.schline_text = NULL;
  }
  ^{PT}BOUNDS\.                     read_bounds(schline.bounds);
  ^{PT}SCHLINE_ATTR\.                yy_push_state(SCHLINE_ATTR);
  ^#END_SCHLINE  {
    write_schline(&schline, binfile);
    write_header((int)schline.identification.ui_object_id);
    {int len; len=ftell(binfile); printf(" end=0x%06x %i\n",len,len);}
    BEGIN(INITIAL);
  }
  .  error("SCHLINE");
}

<SCHLINE_ATTR>{
  (SCHLINE_)?ENABLE{EQ}   { yy_pop_state(); read_boolean(schline.schline_attr.schline_enable); }
  (SCHLINE_)?ACTIVE{EQ}   { yy_pop_state(); read_boolean(schline.schline_attr.schline_active); }
  (SCHLINE_)?VISIBLE{EQ}  { yy_pop_state(); read_boolean(schline.schline_attr.schline_visible); }
  .                       error("SCHLINE_ATTR");
}

<STRING>{
  ^I[D]?{EQ}                        read_num(&string.identification.ui_object_id);
  ^{PT}OBJECT\.TYPE{EQ}STRING{NL} {
    printf("    ID=%-4i",string.identification.ui_object_id);
    string.identification.ui_object_type = 8;
    string.string_text = NULL;
  }
  ^{PT}RELATED_TABLE_ID{EQ}         read_num(&tableid);
  ^{PT}BOUNDS\.                     read_bounds(string.bounds);
  ^{PT}STRING_STYLE{EQ}             yy_push_state(STRING_STYLE);
  ^{PT}STRING.TEXT{EQ}              read_text(string.string_text);
  ^{PT}STRING_(FONT_)?COLOR{EQ}     read_color(string.string_color);
  ^{PT}STRING_B[A-Z]*_COLOR{EQ}     read_color(string.string_bg_color);
  ^{PT}STRING_FONT{EQ}              read_font(string.text_font);
  ^{PT}STRING_[A-Z]*MENT{EQ}        read_alignment(string.text_alignment);
  ^{PT}STRING_ATTR\.                yy_push_state(STRING_ATTR);
  ^#END_STRING  {
    write_string(&string, tableid, binfile);
    write_header((int)string.identification.ui_object_id);
    {int len; len=ftell(binfile); printf(" end=0x%06x %i\n",len,len);}
    BEGIN(INITIAL);
  }
  .  error("STRING");
}

<STRING_STYLE>{
  [A-Z_]*/[0-9]       { yy_pop_state(); read_num(&string.string_style); }
  .                   error("STRING_STYLE");
}

<STRING_ATTR>{
  (STRING_)?VISIBLE{EQ}  { yy_pop_state(); read_boolean(string.string_attr.string_visible); }
  .                      error("STRING_ATTR");
}

<BITMAP>{
  ^I[D]?{EQ}                       read_num(&bitmap.identification.ui_object_id);
  ^{PT}OBJECT\.TYPE{EQ}BITMAP{NL}  {
    printf("    ID=%-4i",bitmap.identification.ui_object_id);
    bitmap.identification.ui_object_type = 9;
    bitmapnum = -4;
    bitmap.bitmap_bitmap1.bitmap_data = NULL;
    bitmap.bitmap_bitmap2.bitmap_data = NULL;
  }
  ^{PT}RELATED_TABLE_ID{EQ}        read_num(&tableid);
  ^{PT}BOUNDS\. {
    if(bitmapnum<0) {
      read_bounds(bitmap.bounds);
      ++bitmapnum;
    } else if(bitmapnum==0) {
      read_bitmap(bitmap.bitmap_bitmap1);
    } else if(bitmapnum==1) {
      read_bitmap(bitmap.bitmap_bitmap2);
    } else {
      fprintf(stderr,"error bitmapnum=%i\n",bitmapnum);
      exit(1);
    }
  }
  ^{PT}BITMAP[._]STYLE{EQ}         yy_push_state(BITMAP_STYLE);
  ^{PT}BITMAP[0-9]?_BITMAP\.  {
    if(bitmapnum==0) {
      read_bitmap(bitmap.bitmap_bitmap1);
    } else {
      read_bitmap(bitmap.bitmap_bitmap2);
    }
  }
  ^{PT}BITMAP_ATTR\.               yy_push_state(BITMAP_ATTR);
  ^#END_BITMAP {
    write_bitmap(&bitmap, tableid, binfile);
    write_header((int)bitmap.identification.ui_object_id);
    {int len; len=ftell(binfile); printf(" end=0x%06x %i\n",len,len);}
    BEGIN(INITIAL);
  }
  .  error("BITMAP");
}

<BITMAP_STYLE>{
  [A-Z_]*/[0-9]  { yy_pop_state(); read_num(&bitmap.bitmap_style); }
  .              error("BITMAP_STYLE");
}

<BITMAP_ATTR>{
  BITMAP_ENABLE{EQ}   { yy_pop_state(); read_boolean(bitmap.bitmap_attr.bitmap_enable); }
  BITMAP_ACTIVE{EQ}   { yy_pop_state(); read_boolean(bitmap.bitmap_attr.bitmap_active); }
  BITMAP_VISIBLE{EQ}  { yy_pop_state(); read_boolean(bitmap.bitmap_attr.bitmap_visible); }
  .                   error("BITMAP_ATTR");
}

<TEXTBOX>{
  ^I[D]?{EQ}                                 read_num(&textbox.identification.ui_object_id);
  ^{PT}OBJECT\.TYPE{EQ}TEXTBOX{NL} {
    printf("   ID=%-4i",textbox.identification.ui_object_id);
    textbox.identification.ui_object_type = 10;
  }
  ^{PT}RELATED_TABLE_ID{EQ}                  read_num(&tableid);
  ^{PT}BOUNDS[_.]                            read_bounds(textbox.bounds);
  ^{PT}TEXTBOX_TEXT{EQ}                      read_text(textbox.textbox_string);
  ^{PT}TEXTBOX_STYLE{EQ}                     yy_push_state(TEXTBOX_STYLE);
  ^{PT}TEXTBOX_BACK_LINE{EQ}                 yy_push_state(TEXTBOX_BACK_LINE);
  ^{PT}TEXTBOX_FONT{EQ}                      read_font(textbox.textbox_font_id);
  ^{PT}TEXTBOX_FONT_COLOR{EQ}                read_color(textbox.textbox_font_color);
  ^{PT}TEXTBOX_BACKGROUND_COLOR{EQ}          read_color(textbox.textbox_background_color);
  ^{PT}TEXTBOX_MAX_NUM_CHARS{EQ}             read_num(&textbox.textbox_max_chars);
  ^{PT}TEXTBOX_LEFT_CHAR_POS{EQ}             read_num(&textbox.textbox_left_char_pos);
  ^{PT}TEXTBOX_RIGHT_CHAR_POS{EQ}            read_num(&textbox.textbox_right_char_pos);
  ^{PT}TEXTBOX_INSERT_PT_CHAR_POS{EQ}        read_num(&textbox.textbox_insert_pt_char_pos);
  ^{PT}TEXTBOX_HIGHLIGHT_START_CHAR_POS{EQ}  read_num(&textbox.textbox_highlight_start_char);
  ^{PT}TEXTBOX_HIGHLIGHT_END_CHAR_POS{EQ}    read_num(&textbox.textbox_highlight_end_char);
  ^{PT}TEXTBOX_ATTR[_.]                      yy_push_state(TEXTBOX_ATTR);
  ^#END_TEXTBOX {
    write_textbox(&textbox, tableid, binfile);
    write_header((int)textbox.identification.ui_object_id);
    {int len; len=ftell(binfile); printf(" end=0x%06x %i\n",len,len);}
    BEGIN(INITIAL);
  }
  .  error("TEXTBOX");
}

<TEXTBOX_STYLE>{
  [A-Z_]*/[0-9]  { yy_pop_state(); read_num(&textbox.textbox_style); }
  .              error("TEXTBOX_STYLE");
}

<TEXTBOX_BACK_LINE>{
  NO[A-Z_]*{NL}    { textbox.textbox_back_line = 0; yy_pop_state(); }
  DOT[A-Z_]*{NL}   { textbox.textbox_back_line = 1; yy_pop_state(); }
  GREY[A-Z_]*{NL}  { textbox.textbox_back_line = 2; yy_pop_state(); }
  .                error("TEXTBOX_BACK_LINE");
}

<TEXTBOX_ATTR>{
  TEXTBOX_ACTIVE{EQ}   { yy_pop_state(); read_boolean(textbox.textbox_attr.textbox_active); }
  TEXTBOX_ENABLE{EQ}   { yy_pop_state(); read_boolean(textbox.textbox_attr.textbox_enable); }
  TEXTBOX_DIRTY{EQ}  { yy_pop_state(); read_boolean(textbox.textbox_attr.textbox_dirty); }
  TEXTBOX_HIGHLIGHT{EQ}  { yy_pop_state(); read_boolean(textbox.textbox_attr.textbox_highlight); }
  TEXTBOX_INSERT_PT_VISIBLE{EQ}  { yy_pop_state(); read_boolean(textbox.textbox_attr.textbox_insert_pt_visible); }
  TEXTBOX_VISIBLE{EQ}  { yy_pop_state(); read_boolean(textbox.textbox_attr.textbox_visible); }
  .                    error("TEXTBOX_ATTR");
}

<LINE>{
  ^I[D]?{EQ}                        read_num(&line.identification.ui_object_id);
  ^{PT}OBJECT\.TYPE{EQ}LINE{NL} {
    printf("      ID=%-4i",line.identification.ui_object_id);
    line.identification.ui_object_type = 11;
  }
  ^{PT}RELATED_TABLE_ID{EQ}         read_num(&tableid);
  ^{PT}BOUNDS\.                     read_bounds(line.bounds);
  ^{PT}LINE_COLOR{EQ}               read_color(line.line_color);
  ^{PT}LINE_STYLE{EQ}               yy_push_state(LINE_STYLE);
  ^{PT}LINE_THICK{EQ}               read_num(&line.line_thick);
  ^{PT}LINE_ENDPT_X{EQ}             read_num(&line.line_endpt_xpos);
  ^{PT}LINE_ENDPT_Y{EQ}             read_num(&line.line_endpt_ypos);
  ^{PT}LINE_ATTR\.LINE_VISIBLE{EQ}  read_boolean(line.line_attr.line_visible);
  ^#END_LINE {
    write_line(&line, tableid, binfile);
    write_header((int)line.identification.ui_object_id);
    {int len; len=ftell(binfile); printf(" end=0x%06x %i\n",len,len);}
    BEGIN(INITIAL);
  }
  .  error("LINE");
}

<LINE_STYLE>{
  DOTTED_LINE{NL}     { line.line_style = 0; yy_pop_state(); }
  NON_DOTTED_LINE{NL} { line.line_style = 1; yy_pop_state(); }
  [A-Z_]*/[0-9]       { yy_pop_state(); read_num(&line.line_style); }
  .                   error("LINE_STYLE");
}

%%

void read_num(BYTE *x)   { rbyte   = x; yy_push_state(READ_BYTE); }
void read_num(UBYTE *x)  { rubyte  = x; yy_push_state(READ_UBYTE); }
void read_num(SHORT *x)  { rshort  = x; yy_push_state(READ_SHORT); }
void read_num(USHORT *x) { rushort = x; yy_push_state(READ_USHORT); }
void read_num(WORD *x)   { rword   = x; yy_push_state(READ_WORD); }
void read_num(UWORD *x)  { ruword  = x; yy_push_state(READ_UWORD); }

void
error(char *s) {
  int c;
  printf("error: %s\n", s);
  ECHO;
  do {
    c = yyinput();
    putchar(c);
  } while(c!='\n');
  exit(1);
}

void
write_header(int id)
{
  fprintf(hfile,"#define %-40s %i\n",name,id);
  free(name);
}

int
main(int argc, char *argv[])
{
  int size=0;
  FILE *infile;

  printf("Resource Compiler for Helio version 1.0\n");
  printf("This program uses parts from the VTech Helio SDK,\n");
  printf("however it was not created by VTech.\n");
  putchar('\n');

  if(argc!=2) {
    printf("Usage:\n");
    printf("%s <resource text file>\n", argv[0]);
    //putchar('\n');
    printf("generates resource.bin and resource.h\n");
    exit(1);
  }

  infile = fopen(argv[1], "r");
  if(!infile) {
    printf("Can't open file %s\n", argv[1]);
    exit(2);
  }

  binfile = fopen("resource.bin", "w");
  if(!binfile) {
    printf("Can't open output file resource.bin\n");
    exit(3);
  }

  hfile = fopen("resource.h", "w");
  if(!hfile) {
    printf("Can't open output file resource.h\n");
    exit(4);
  }

  size = 0; /* fill it in later */
  fwrite(&size, 4, 1, binfile);

  yyin = infile;
  yylex();

  size = ftell(binfile);
  fseek(binfile, 0, SEEK_SET);
  fwrite(&size, 4, 1, binfile);

  fclose(infile);
  fclose(binfile);
  fclose(hfile);

  return 0;
}
