#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include "ui.h"
#include "reslib.h"

#define write_size(x) write_word(binfile, x)
#define write_data(x) { write_bytes(binfile, (void *)&x, sizeof(x));}
#define write_text(x) write_text1(binfile, x);
#define BEGIN_RESOURCE { \
    int rpos0, rpos1; \
    rpos0 = ftell(binfile); \
    write_size(0);
#define END_RESOURCE \
    rpos1 = ftell(binfile); \
    fseek(binfile, rpos0, SEEK_SET); \
    write_size(rpos1-rpos0); \
    fseek(binfile, rpos1, SEEK_SET); \
  }
#define BEGIN_FIELD { \
    int fpos0, fpos1; \
    fpos0 = ftell(binfile); \
    write_size(0);
#define END_FIELD \
    fpos1 = ftell(binfile); \
    fseek(binfile, fpos0, SEEK_SET); \
    write_size(fpos1-fpos0); \
    fseek(binfile, fpos1, SEEK_SET); \
  }
#define NEW_FIELD END_FIELD BEGIN_FIELD

void
write_bytes(FILE *f, void *x, int len)
{
  fwrite(x, len, 1, f);
}

void
write_byte(FILE *f, BYTE x)
{
  fwrite(&x, 1, 1, f);
}

void
write_ubyte(FILE *f, UBYTE x)
{
  fwrite(&x, 1, 1, f);
}

void
write_short(FILE *f, SHORT x)
{
  fwrite(&x, 2, 1, f);
}

void
write_ushort(FILE *f, USHORT x)
{
  fwrite(&x, 2, 1, f);
}

void
write_word(FILE *f, WORD x)
{
  fwrite(&x, 4, 1, f);
}

void
write_uword(FILE *f, UWORD x)
{
  fwrite(&x, 4, 1, f);
}

void
write_boolean(FILE *f, BOOLEAN x)
{
  fwrite(&x, 2, 1, f);
}

void
write_text1(FILE *f, BYTE *x)
{
  if(x==NULL) {
    //fputc(0, binfile);
  } else {
    int len;
    len = strlen(x) + 1;
    fwrite(x, 1, len, f);
  }
}

void
write_bounds(FILE *f, ObjectBounds *b)
{
  write_short(f, b->xcoord);
  write_short(f, b->ycoord);
  write_short(f, b->width);
  write_short(f, b->height);
}

void
write_bitmap_template(FILE *binfile, BitmapTemplate *bt)
{
  int i;
  int bitmappad, bitmapsize;

  if(bt->bitmap_data==NULL) return;

  bitmappad = 3 - ((ftell(binfile)-1)%4);
  bitmapsize = 16 + bitmappad + bt->size;

  write_short(binfile, bt->xcoord);
  write_short(binfile, bt->ycoord);
  write_short(binfile, bt->width);
  write_short(binfile, bt->height);
  write_byte(binfile, bt->compressed);
  write_byte(binfile, bt->quantisation);
  write_short(binfile, bt->size);
  for(i=0; i<bitmappad; ++i) fputc(0, binfile);
  for(i=0; i<bt->size/4; ++i) {
    write_uword(binfile, bt->bitmap_data[i]);
  }
}

void
load_bitmap_template(BitmapTemplate *bt, char *filename)
{
  int colorindex[] = { 1, 2, 3,  4,  62,  63,  64,  80,
		       96, 106, 108, 144, 156, 168, 180, 256 };
  int filesize, dataoffset, width, height, compression, ncolors;
  int i;
  int inbytesperline, ninbytes, ninbits;
  int outwordsperline, noutwords, noutbits;
  int inbyte, index, color;
  UWORD outword;
  short bitcount;
  char magic[2];
  unsigned char *r, *g, *b;
  FILE *bmpf;

  //printf("bitmapfilename: %s\n", filename);
  bmpf = fopen(filename, "rb");
  if(!bmpf) {
    printf("error: can't open file %s\n", filename);
    exit(1);
  }

  fread(&magic, 1, 2, bmpf);
  fread(&filesize, 4, 1, bmpf);
  fseek(bmpf, 4, SEEK_CUR);
  fread(&dataoffset, 4, 1, bmpf);
  fseek(bmpf, 4, SEEK_CUR);
  fread(&width, 4, 1, bmpf);
  fread(&height, 4, 1, bmpf);
  fseek(bmpf, 2, SEEK_CUR);
  fread(&bitcount, 2, 1, bmpf);
  fread(&compression, 4, 1, bmpf);
  fseek(bmpf, 20, SEEK_CUR);

  ncolors = 1<<bitcount;
  r = (unsigned char *)malloc(ncolors);
  g = (unsigned char *)malloc(ncolors);
  b = (unsigned char *)malloc(ncolors);
  if(!r) {
    printf("error: no mem for R file %s\n", filename);
    exit(1);
  }
  if(!g) {
    printf("error: no mem for G file %s\n", filename);
    exit(1);
  }
  if(!b) {
    printf("error: no mem for B file %s\n", filename);
    exit(1);
  }

  for(i=0; i<ncolors; ++i) {
    r[i] = fgetc(bmpf);
    g[i] = fgetc(bmpf);
    b[i] = fgetc(bmpf);
    fgetc(bmpf);
  }

  fseek(bmpf, dataoffset, SEEK_SET);

  inbytesperline = 4*(((width*bitcount)+31)/32);
  outwordsperline = (((bt->width*bt->quantisation)+31)/32);
  bt->size = 4*outwordsperline*height;
  bt->bitmap_data = (UWORD *)malloc(bt->size);
  if(!bt->bitmap_data) {
    printf("error: no mem for file %s\n", filename);
    exit(1);
  }

  for(i=0; i<bt->height; ++i) {
    ninbytes = 0;
    ninbits = 8;
    outword = 0;
    noutwords = 0;
    noutbits = 0;
    do {
      if(ninbytes<inbytesperline) {
	if(ninbits>=8) {
	  inbyte = fgetc(bmpf);
	  ++ninbytes;
	  ninbits = 0;
	}
	index = (inbyte >> (8-bitcount*(ninbits+1)))&((1<<bitcount)-1);
	//printf("%x ",(int)r[index]);
	ninbits += bitcount;
	//color = ((r[index]+g[index]+b[index])*((1<<bt->quantisation)-0)) /
	//(3<<bitcount);
	color = (r[index]+g[index]+b[index])/3;
	color = 255 - color;
	//printf("%x ", color);
	index = 0;
	while(color>colorindex[index]) ++index;
	color = (index*(1<<bt->quantisation))/16;
	color &= (1<<bt->quantisation)-1;
	//color ^= (1<<bt->quantisation)-1;
      } else {
	if(ninbits>=0) ninbits = -1;
	else --ninbits;
	if(ninbits==-1) {
	  color = (1<<bt->quantisation)-1;
	} else if(ninbits==-2) {
	  color = (1<<bt->quantisation)-1;
	} else {
	  color = 0;
	}
	color = 0;
      }

      if(noutwords<outwordsperline) {
	outword += color;
	noutbits += bt->quantisation;
	if(noutbits>=32) {
	  bt->bitmap_data[(bt->height-1-i)*outwordsperline+noutwords] =outword;
	  ++noutwords;
	  outword = 0;
	  noutbits = 0;
	} else {
	  outword <<= bt->quantisation;
	}
      }
    } while((ninbytes<inbytesperline)||(noutwords<outwordsperline));
  }
  fclose(bmpf);
  free(r);
  free(g);
  free(b);
}

void
print_id(Identification *id)
{
  printf("%i %i\n", id->ui_object_id, id->ui_object_type);
}

void
print_bounds(ObjectBounds *bounds)
{
  printf("%i %i %i %i\n", bounds->xcoord, bounds->ycoord, bounds->width,
	 bounds->height);
}

void
print_bitmap_template(BitmapTemplate *bt)
{
  printf("%i %i %i %i\n", bt->xcoord, bt->ycoord, bt->width, bt->height);
  printf("%i %i %i\n", bt->compressed, bt->quantisation, bt->size);
}

void
write_form(Form *form, FILE *binfile)
{
  int i;

  write_data(form->identification.ui_object_id);
  BEGIN_RESOURCE;
  BEGIN_FIELD; //0
  write_data(form->identification.ui_object_type);
  NEW_FIELD; //1
  write_bounds(binfile, &form->bounds);
  NEW_FIELD; //2
  write_data(form->form_focus);
  NEW_FIELD; //3
  write_data(form->form_style);
  NEW_FIELD; //4
  write_text(form->form_dialog_title);
  NEW_FIELD; //5
  if((form->form_style==1)||(form->form_style==2))
    write_bitmap_template(binfile, &form->form_bitmap);
  NEW_FIELD; //6
  write_data(form->form_num_objects);
  NEW_FIELD; //7
  for(i=0; i<form->form_num_objects; ++i) {
    write_data(form->form_objects_list[i].object_id);
    write_data(form->form_objects_list[i].object_type);
  }
  END_FIELD;
  END_RESOURCE;
}

void
write_control(Control *control, void *control_add, USHORT tableid, FILE *binfile)
{
  int i;

  write_data(control->identification.ui_object_id);
  BEGIN_RESOURCE;
  BEGIN_FIELD; //0
  write_data(control->identification.ui_object_type);
  NEW_FIELD; //1
  write_data(tableid);
  NEW_FIELD; //2
  write_data(control->bounds);
  NEW_FIELD; //3
  write_data(control->control_style);
  NEW_FIELD; //4
  if(control->control_style!=3) write_data(control->control_subtype);
  NEW_FIELD; //5
  write_text(control->control_text);
  NEW_FIELD; //6
  write_data(control->text_alignment);
  NEW_FIELD; //7
  write_data(control->control_attr.control_enable);
  write_data(control->control_attr.control_active);
  write_data(control->control_attr.control_visible);
  END_FIELD;
  switch(control->control_style) {
  case 0:
    BEGIN_FIELD; //8
    write_data(((ControlTemplateButton*)control_add)->button_radius);
    write_data(((ControlTemplateButton*)control_add)->button_color_on);
    write_data(((ControlTemplateButton*)control_add)->button_color_off);
    END_FIELD;
    break;
  case 1:
    BEGIN_FIELD; //8
    write_data(((ControlTemplatePushButton*)control_add)->push_button_color_on);
    write_data(((ControlTemplatePushButton*)control_add)->push_button_color_off);
    write_data(((ControlTemplatePushButton*)control_add)->push_button_group_id);
    write_data(((ControlTemplatePushButton*)control_add)->control_value);
    write_data(((ControlTemplatePushButton*)control_add)->push_button_radius);
    END_FIELD;
    break;
  case 2:
    BEGIN_FIELD; //8
    write_bitmap_template(binfile, &((ControlTemplateRepeatButton*)control_add)->repeat_bitmap);
    END_FIELD;
    break;
  case 3:
    BEGIN_FIELD; //8
    write_data(((ControlTemplateCheckBox*)control_add)->checkbox_group_id);
    write_data(((ControlTemplateCheckBox*)control_add)->control_value);
    write_bitmap_template(binfile, &((ControlTemplateCheckBox*)control_add)->checkbox_bitmap1);
    NEW_FIELD; //9
    write_bitmap_template(binfile, &((ControlTemplateCheckBox*)control_add)->checkbox_bitmap2);
    END_FIELD;
    break;
  case 4:
    BEGIN_FIELD; //8
    write_data(((ControlTemplatePopupTrigger*)control_add)->popup_radius);
    write_data(((ControlTemplatePopupTrigger*)control_add)->popup_num_objects);
    write_data(((ControlTemplatePopupTrigger*)control_add)->popup_selected_item);
    write_data(((ControlTemplatePopupTrigger*)control_add)->popup_top_item_num);
    for(i=0; i<((ControlTemplatePopupTrigger*)control_add)->popup_num_objects; ++i) {
      write_text(((ControlTemplatePopupTrigger*)control_add)->popup_items_list[i]);
    }
    END_FIELD;
    break;
  default: fprintf(stderr,"write_control unknown style %i\n",control->control_style); exit(1);
  }
  END_RESOURCE;
}

void
write_field(Field *field, USHORT tableid, FILE *binfile)
{
  write_data(field->identification.ui_object_id);
  BEGIN_RESOURCE;
  BEGIN_FIELD; //0
  write_data(field->identification.ui_object_type);
  NEW_FIELD; //1
  write_data(tableid);
  NEW_FIELD; //2
  write_data(field->bounds);
  NEW_FIELD; //3
  write_text(field->field_string);
  NEW_FIELD; //4
  write_data(field->field_style);
  NEW_FIELD; //5
  write_data(field->field_back_line);
  NEW_FIELD; //6
  write_data(field->field_font_id);
  NEW_FIELD; //7
  write_data(field->field_font_color);
  NEW_FIELD; //8
  write_data(field->field_background_color);
  NEW_FIELD; //9
  write_data(field->field_text_alignment);
  NEW_FIELD; //10
  write_data(field->field_max_chars);
  NEW_FIELD; //11
  write_data(field->field_top_line_num);
  NEW_FIELD; //12
  write_data(field->field_insert_pt_char_pos);
  NEW_FIELD; //13
  write_data(field->field_highlight_start_char);
  NEW_FIELD; //14
  write_data(field->field_highlight_end_char);
  NEW_FIELD; //15
  write_data(field->field_attr.field_active);
  write_data(field->field_attr.field_enable);
  write_data(field->field_attr.field_dirty);
  write_data(field->field_attr.field_highlight);
  write_data(field->field_attr.field_insert_pt_visible);
  write_data(field->field_attr.field_scrollbar);
  write_data(field->field_attr.field_full_size);
  write_data(field->field_attr.field_visible);
  END_FIELD;
  END_RESOURCE;
}

void
write_list(List *list, USHORT tableid, FILE *binfile)
{
  int i;

  write_data(list->identification.ui_object_id);
  BEGIN_RESOURCE;
  BEGIN_FIELD; //0
  write_data(list->identification.ui_object_type);
  NEW_FIELD; //1
  write_data(tableid);
  NEW_FIELD; //2
  write_data(list->bounds);
  NEW_FIELD; //3
  write_data(list->list_num_related_list);
  NEW_FIELD; //4
  for(i=0; i<list->list_num_related_list; ++i)
    write_data(list->list_related_list_id[i]);
  NEW_FIELD; //5
  write_data(list->list_total_num_items);
  NEW_FIELD; //6
  write_data(list->list_num_items_on_display);
  NEW_FIELD; //7
  write_data(list->list_top_item_num);
  NEW_FIELD; //8
  write_data(list->list_text_alignment);
  NEW_FIELD; //9
  write_data(list->list_selected_item);
  NEW_FIELD; //10
  write_data(list->list_style);
  NEW_FIELD; //11
  write_data(list->list_text_color);
  NEW_FIELD; //12
  write_data(list->list_bg_color);
  NEW_FIELD; //13
  for(i=0; i<list->list_total_num_items; ++i)
    write_text(list->list_items[i]);
  NEW_FIELD; //14
  write_data(list->list_attr.list_enable);
  write_data(list->list_attr.list_visible);
  write_data(list->list_attr.list_active);
  write_data(list->list_attr.list_set_scroll);
  write_data(list->list_attr.list_synchronous);
  END_FIELD;
  END_RESOURCE;
}

void
write_menu(Menu *menu, FILE *binfile)
{
  int i;

  write_data(menu->identification.ui_object_id);
  BEGIN_RESOURCE;
  BEGIN_FIELD; //0
  write_data(menu->identification.ui_object_type);
  NEW_FIELD; //1
  write_data(menu->bounds);
  NEW_FIELD; //2
  write_data(menu->menu_num_items);
  NEW_FIELD; //3
  write_data(menu->menu_selected_item);
  NEW_FIELD; //4
  for(i=0; i<menu->menu_num_items; ++i) write_text(menu->menu_items[i]);
  NEW_FIELD; //5
  write_data(menu->menu_attr.menu_visible);
  END_FIELD;
  END_RESOURCE;
}

void
write_scrollbar(Scrollbar *scrollbar, USHORT tableid, FILE *binfile)
{
  write_data(scrollbar->identification.ui_object_id);
  BEGIN_RESOURCE;
  BEGIN_FIELD; //0
  write_data(scrollbar->identification.ui_object_type);
  NEW_FIELD; //1
  write_data(tableid);
  NEW_FIELD; //2
  write_data(scrollbar->bounds);
  NEW_FIELD; //3
  write_data(scrollbar->scrollbar_max);
  NEW_FIELD; //4
  write_data(scrollbar->scrollbar_min);
  NEW_FIELD; //5
  write_data(scrollbar->scrollbar_value);
  NEW_FIELD; //6
  write_data(scrollbar->scrollbar_pagesize);
  NEW_FIELD; //7
  write_data(scrollbar->scrollbar_total_num_lines);
  NEW_FIELD; //8
  write_data(scrollbar->scrollbar_style);
  NEW_FIELD; //9
  write_data(scrollbar->scrollbar_type);
  NEW_FIELD; //10
  write_bitmap_template(binfile, &scrollbar->scrollbar_arrow1);
  NEW_FIELD; //11
  write_bitmap_template(binfile, &scrollbar->scrollbar_arrow2);
  NEW_FIELD; //12
  write_text(scrollbar->scrollbar_text);
  NEW_FIELD; //13
  write_data(scrollbar->scrollbar_attr.scrollbar_active);
  write_data(scrollbar->scrollbar_attr.scrollbar_enable);
  write_data(scrollbar->scrollbar_attr.scrollbar_visible);
  END_FIELD;
  END_RESOURCE;
}

void
write_table(Table *table, FILE *binfile)
{
  int i, num;

  write_data(table->identification.ui_object_id);
  BEGIN_RESOURCE;
  BEGIN_FIELD; //0
  write_data(table->identification.ui_object_type);
  NEW_FIELD; //1
  write_data(table->bounds);
  NEW_FIELD; //2
  write_data(table->table_num_column);
  NEW_FIELD; //3
  write_data(table->table_num_row);
  NEW_FIELD; //4
  for(i=0; i<table->table_num_column; ++i) write_data(table->table_column_width[i]);
  NEW_FIELD; //5
  for(i=0; i<table->table_num_row; ++i) write_data(table->table_row_height[i]);
  NEW_FIELD; //6
  write_data(table->table_top_row_num);
  NEW_FIELD; //7
  write_data(table->table_left_col_num);
  NEW_FIELD; //8
  write_data(table->table_style);
  NEW_FIELD; //9
  write_data(table->table_bg_color);
  NEW_FIELD; //10
  write_bitmap_template(binfile, &table->table_cell_bitmap);
  NEW_FIELD; //11
  num = table->table_num_column * table->table_num_row;
  for(i=0; i<num; ++i) {
    write_data(table->table_item_ptr[i]->table_data_type);
    write_data(table->table_item_ptr[i]->table_item_ui_id);
    write_data(table->table_item_ptr[i]->table_font);
    write_data(table->table_item_ptr[i]->table_value);
    write_data(table->table_item_ptr[i]->table_display_alignment);
    write_data(table->table_item_ptr[i]->table_text_color);
    write_data(table->table_item_ptr[i]->table_text_bg_color);
    write_data(table->table_item_ptr[i]->table_cell_highlight);
    write_data(table->table_item_ptr[i]->table_cell_has_bitmap);
    write_text(table->table_item_ptr[i]->table_text);
  }
  NEW_FIELD; //12
  write_data(table->table_attr.table_enable);
  write_data(table->table_attr.table_scrollbar);
  write_data(table->table_attr.table_active);
  write_data(table->table_attr.table_highlight_enable);
  write_data(table->table_attr.table_visible);
  END_FIELD;
  END_RESOURCE;
}

void
write_schline(Schline *schline, FILE *binfile)
{
  write_data(schline->identification.ui_object_id);
  BEGIN_RESOURCE;
  BEGIN_FIELD; //0
  write_data(schline->identification.ui_object_type);
  NEW_FIELD; //1
  write_data(schline->bounds);
  NEW_FIELD; //2
  write_data(schline->schline_attr.schline_enable);
  write_data(schline->schline_attr.schline_active);
  write_data(schline->schline_attr.schline_visible);
  END_FIELD;
  END_RESOURCE;
}

void
write_string(String *string, USHORT tableid, FILE *binfile)
{
  write_data(string->identification.ui_object_id);
  BEGIN_RESOURCE;
  BEGIN_FIELD; //0
  write_data(string->identification.ui_object_type);
  NEW_FIELD; //1
  write_data(tableid);
  NEW_FIELD; //2
  write_data(string->bounds);
  NEW_FIELD; //3
  write_data(string->string_style);
  NEW_FIELD; //4
  write_text(string->string_text);
  NEW_FIELD; //5
  write_data(string->string_color);
  NEW_FIELD; //6
  write_data(string->string_bg_color);
  NEW_FIELD; //7
  write_data(string->text_font);
  NEW_FIELD; //8
  write_data(string->text_alignment);
  NEW_FIELD; //9
  write_data(string->string_attr.string_visible);
  END_FIELD;
  END_RESOURCE;
}

void
write_bitmap(Bitmap *bitmap, USHORT tableid, FILE *binfile)
{
  write_data(bitmap->identification.ui_object_id);
  BEGIN_RESOURCE;
  BEGIN_FIELD;
  write_data(bitmap->identification.ui_object_type);
  NEW_FIELD;
  write_data(tableid);
  NEW_FIELD;
  write_data(bitmap->bounds);
  NEW_FIELD;
  write_data(bitmap->bitmap_style);
  NEW_FIELD;
  write_bitmap_template(binfile, &bitmap->bitmap_bitmap1);
  NEW_FIELD;
  write_bitmap_template(binfile, &bitmap->bitmap_bitmap2);
  NEW_FIELD;
  write_data(bitmap->bitmap_attr.bitmap_enable);
  write_data(bitmap->bitmap_attr.bitmap_active);
  write_data(bitmap->bitmap_attr.bitmap_visible);
  END_FIELD;
  END_RESOURCE;
}

void
write_textbox(Textbox *textbox, USHORT tableid, FILE *binfile)
{
  write_data(textbox->identification.ui_object_id);
  BEGIN_RESOURCE;
  BEGIN_FIELD; //0
  write_data(textbox->identification.ui_object_type);
  NEW_FIELD; //1
  write_data(tableid);
  NEW_FIELD; //2
  write_data(textbox->bounds);
  NEW_FIELD; //3
  write_text(textbox->textbox_string);
  NEW_FIELD; //4
  write_data(textbox->textbox_style);
  NEW_FIELD; //5
  write_data(textbox->textbox_back_line);
  NEW_FIELD; //6
  write_data(textbox->textbox_font_id);
  NEW_FIELD; //7
  write_data(textbox->textbox_font_color);
  NEW_FIELD; //8
  write_data(textbox->textbox_background_color);
  NEW_FIELD; //9
  write_data(textbox->textbox_max_chars);
  NEW_FIELD; //10
  write_data(textbox->textbox_left_char_pos);
  NEW_FIELD; //11
  write_data(textbox->textbox_right_char_pos);
  NEW_FIELD; //12
  write_data(textbox->textbox_insert_pt_char_pos);
  NEW_FIELD; //13
  write_data(textbox->textbox_highlight_start_char);
  NEW_FIELD; //14
  write_data(textbox->textbox_highlight_end_char);
  NEW_FIELD; //15
  write_data(textbox->textbox_attr.textbox_active);
  write_data(textbox->textbox_attr.textbox_enable);
  write_data(textbox->textbox_attr.textbox_dirty);
  write_data(textbox->textbox_attr.textbox_highlight);
  write_data(textbox->textbox_attr.textbox_insert_pt_visible);
  write_data(textbox->textbox_attr.textbox_visible);
  END_FIELD;
  END_RESOURCE;
}

void
write_line(Line *line, USHORT tableid, FILE *binfile)
{
  write_data(line->identification.ui_object_id);
  BEGIN_RESOURCE;
  BEGIN_FIELD; //0
  write_data(line->identification.ui_object_type);
  NEW_FIELD; //1
  write_data(tableid);
  NEW_FIELD; //2
  write_data(line->bounds);
  NEW_FIELD; //3
  write_data(line->line_color);
  NEW_FIELD; //4
  write_data(line->line_style);
  NEW_FIELD; //5
  write_data(line->line_thick);
  NEW_FIELD; //6
  write_data(line->line_endpt_xpos);
  NEW_FIELD; //7
  write_data(line->line_endpt_ypos);
  NEW_FIELD; //8
  write_data(line->line_attr.line_visible);
  END_FIELD;
  END_RESOURCE;
}
